/*
   This file defines a class Hat.

   This version of the Hat class presents the exact
   same public interface as the other version of the
   Hat class. But this version has a very different
   way of storing the information about a hat. This
   version uses a single integer, itemKey, to store the
   information about the hat's size, style, and color.
   The size is put in the ones digit of itemKey, the
   style is put in the tens digit, and the color is in
   the hundreds digit of itemKey.
*/

public class Hat
{
   // a single instance field
   private int itemKey;

   // some size constants
   public static final int SMALL = 1;
   public static final int MEDIUM = 2;
   public static final int LARGE = 3;
   // some style constants
   public static final int BASEBALL = 1;
   public static final int BEANIE = 2;
   // some color constants
   public static final int WHITE = 1;
   public static final int RED = 2;
   public static final int BLUE = 3;

   // default constructor
   public Hat()
   {
      this(MEDIUM, BASEBALL, WHITE);
   }

   // a full constructor
   public Hat(int size, int style, int color)
   {
      itemKey = color*100 + style*10 + size;
   }

   public void setSize(int size)
   {
      itemKey = getColor()*100 + getStyle()*10 + size;
   }

   public void setStyle(int style)
   {
      itemKey = getColor()*100 + style*10 + getSize();
   }

   public void setColor(int color)
   {
      itemKey = color*100 + getStyle()*10 + getSize();
   }

   public int getSize()
   {
      return itemKey%10;
   }

   public int getStyle()
   {
      return (itemKey%100)/10;
   }

   public int getColor()
   {
      return (itemKey%1000)/100;
   }

   public String toString()
   {
      // three local variables
      String sizeString, styleString, colorString;

      // use the three get methods to get the three
      // attributes of this object
      int size = getSize();
      int style = getStyle();
      int color = getColor();

      // determine the appropriate string for the size
      if (size == SMALL) sizeString = "small";
      else if (size == MEDIUM) sizeString = "medium";
      else if (size == LARGE) sizeString = "large";
      else sizeString = "improper"; // should never reach this case!

      // determine the appropriate string for the style
      if (style == BASEBALL) styleString = "baseball";
      else if (style == BEANIE) styleString = "beanie";
      else styleString = "improper"; // should never reach this case!

      // determine the appropriate string for the color
      if (color == WHITE) colorString = "white";
      else if (color == RED) colorString = "red";
      else if (color == BLUE) colorString = "blue";
      else colorString = "improper"; // should never reach this case!

      // return a string that describes this object
      return "size: " + sizeString + "\n"
           + "style: " + styleString + "\n"
           + "color: " + colorString + "\n";
   }

}//Hat