/*
   This class defines an abstract class AbstractGloveItem that
   extends the class OrderItem.

   The public interface defined by this abstract class
   specifies that a glove object has three atributes, a
   size, a style, and a color. These attributes have
   integer values. Symbolic constants are provided
   for the values that these attributes can take.

   This abstract class does not specify how the three
   attributes are to be implemented.
*/

public abstract class AbstractGloveItem extends OrderItem
{
   // some size constants
   public static final int SMALL = 1;
   public static final int MEDIUM = 2;
   public static final int LARGE = 3;
   // some style constants
   public static final int WOMEN = 1;
   public static final int MEN = 2;
   // some color constants
   public static final int BLACK = 1;
   public static final int BROWN = 2;

   // default constructor
   AbstractGloveItem()
   {
      super();
   }

   // a partial constructor that just calls the
   // super class constructor
   AbstractGloveItem(int quantity, double price)
   {
      super(quantity, price);
   }

   public abstract void setSize(int size);

   public abstract void setStyle(int style);

   public abstract void setColor(int color);

   public abstract int getSize();

   public abstract int getStyle();

   public abstract int getColor();

   public String toString()
   {
      // three local variables
      String sizeString, styleString, colorString;

      // use the public interface from this class
      // to get the attributes of an object
      int size = getSize();
      int style = getStyle();
      int color = getColor();

      // determine an appropriate string for the size
      if (size == SMALL) sizeString = "small";
      else if (size == MEDIUM) sizeString = "medium";
      else if (size == LARGE) sizeString = "large";
      else sizeString = "improper"; // should never reach this case!

      // determine an appropriate string for the style
      if (style == WOMEN) styleString = "women";
      else if (style == MEN) styleString = "men";
      else styleString = "improper"; // should never reach this case!

      // determine an appropriate string for the color
      if (color == BLACK) colorString = "black";
      else if (color == BROWN) colorString = "brown";
      else colorString = "improper"; // should never reach this case!

      // return a string that describes an object (notice that the
      // toString() method from the super class is also called)
      return "This is a glove item with\n"
           + "\tsize: " + sizeString + "\n"
           + "\tstyle: " + styleString + "\n"
           + "\tcolor: " + colorString + "\n"
           + super.toString();
   }//toString

}//AbstractGloveItem