/*
   This class defines an abstract class AbstractHatItem that
   extends the class OrderItem.

   The public interface defined by this abstract class
   specifies that a hat object has three atributes, a
   size, a style, and a color. These attributes have
   integer values. Symbolic constants are provided
   for the values that these attributes can take.

   The public interface of this class does not specify how
   the three atributes of a hat object are to be represented
   (i.e., stored) inside a hat object. This implementation
   detail is left to the concrete classes that extend this
   abstract class.
*/

public abstract class AbstractHatItem extends OrderItem
{
   // some size constants
   public static final int SMALL = 1;
   public static final int MEDIUM = 2;
   public static final int LARGE = 3;
   // some style constants
   public static final int BASEBALL = 1;
   public static final int BEANIE = 2;
   // some color constants
   public static final int WHITE = 1;
   public static final int RED = 2;
   public static final int BLUE = 3;

   // default constructor
   AbstractHatItem()
   {
      super();
   }

   // a partial constructor that just calls the
   // super class constructor
   AbstractHatItem(int quantity, double price)
   {
      super(quantity, price);
   }

   public abstract void setSize(int size);

   public abstract void setStyle(int style);

   public abstract void setColor(int color);

   public abstract int getSize();

   public abstract int getStyle();

   public abstract int getColor();

   public String toString()
   {
      // three local variables
      String sizeString, styleString, colorString;

      // use the public interface from this class
      // to get the attributes of an object
      int size = getSize();
      int style = getStyle();
      int color = getColor();

      // determine an appropriate string for the size
      if (size == SMALL) sizeString = "small";
      else if (size == MEDIUM) sizeString = "medium";
      else if (size == LARGE) sizeString = "large";
      else sizeString = "improper"; // should never reach this case!

      // determine an appropriate string for the style
      if (style == BASEBALL) styleString = "baseball";
      else if (style == BEANIE) styleString = "beanie";
      else styleString = "improper"; // should never reach this case!

      // determine an appropriate string for the color
      if (color == WHITE) colorString = "white";
      else if (color == RED) colorString = "red";
      else if (color == BLUE) colorString = "blue";
      else colorString = "improper"; // should never reach this case!

      // return a string that describes an object (notice that the
      // toString() method from the super class is also called)
      return "This is a hat item with\n"
           + "\tsize: " + sizeString + "\n"
           + "\tstyle: " + styleString + "\n"
           + "\tcolor: " + colorString + "\n"
           + super.toString();
   }//toString

}//AbstractHatItem