/*
   The classes in this file provide another demonstration
   of polymorphism and also a demonstration of casting
   reference variables. It is important to notice that
   we cast reference variables, not objects!

   One of the lessons from this example is that you must
   keep separate a reference variable from the object it
   refers to. The reference variable is not the object,
   it just refers to the object.
*/

class SimpleClass1
{
   public String aMethod()
   {
      return("this aMethod in SimpleClass1");
   }
}//SimpleClass1

class SimpleClass2 extends SimpleClass1
{
   public String aMethod()
   {
      return("this is aMethod in SimpleClass2");
   }

   public String bMethod()
   {
      return("this is bMethod in SimpleClass2");
   }
}//SimpleClass2

class SimpleClass3 extends SimpleClass2
{
   public String aMethod()
   {
      return("this is aMethod in SimpleClass3");
   }

   public String bMethod()
   {
      return("this is bMethod in SimpleClass3");
   }

   public String cMethod()
   {
      return("this is cMethod in SimpleClass3");
   }
}//SimpleClass3


public class PolymorphismAndCasting
{
   public static void main(String[] args)
   {
      // some local reference variables
      SimpleClass1 simpleObject1;
      SimpleClass2 simpleObject2;
      SimpleClass3 simpleObject3;

      // create an object
      simpleObject1 = new SimpleClass3();

      // make a polymorphic call to the method aMethod()
      System.out.println( simpleObject1.aMethod() );

      // the following is not allowed (why not?)
      System.out.println( simpleObject1.bMethod() );
      // here is how we can call the method bMethod()
      // the next line combines a cast with a polymorphic method call
      //(what do you think the printout will be?)
      System.out.println( ((SimpleClass2) simpleObject1).bMethod() );

      // or we could do this
      simpleObject2 = (SimpleClass2) simpleObject1;
      System.out.println( simpleObject2.bMethod() );

      // if we want to call cMethod() then we need to do the following
      System.out.println( ((SimpleClass3) simpleObject1).cMethod() );

      // casting of reference variables can be dangerous
      simpleObject1 = new SimpleClass2();
      // the following line, when compiled, will cause a runtime error
      simpleObject3 = (SimpleClass3) simpleObject1;

   }//main

}//PolymorphismAndCasting